

close all;


addpath('/autofs/homes/010/guerin/pTx/Bay5/library');
addpath('/autofs/homes/010/guerin/Utilities_matlab');


% read 3F GRE data
delta_TE=2e-3;  % s
filename1=select_file('*.dat','.','****** Select TE #1 ******');
filename2=select_file('*.dat','.','****** Select TE #2 ******');

meas1=read_meas_dat(filename1);
meas2=read_meas_dat(filename2);

img1=permute( squeeze( mrir_conventional(meas1.data) ),[2 1 4 3] ); img1=img1(end:-1:1,:,end:-1:1,:);
img2=permute( squeeze( mrir_conventional(meas2.data) ),[2 1 4 3] ); img2=img2(end:-1:1,:,end:-1:1,:);

% BC coil combination 
pha=angle( squeeze( img1(end/2,end/2,end/2,:) ) );  % coil combination factors -- BC
[nx ny nz ncoils]=size(img1);
img1_comb=zeros(nx,ny,nz);
img2_comb=zeros(nx,ny,nz);
for i=1:ncoils
    img1_comb=img1_comb + img1(:,:,:,i).*exp(-1j*pha(i));
    img2_comb=img2_comb + img2(:,:,:,i).*exp(-1j*pha(i));
end

to_disp1=[ img1_comb(:,:,1) img1_comb(:,:,2) img1_comb(:,:,3) img1_comb(:,:,4) ;
    img1_comb(:,:,5) img1_comb(:,:,6) img1_comb(:,:,7) img1_comb(:,:,8) ;
    img1_comb(:,:,9) img1_comb(:,:,10) img1_comb(:,:,11) img1_comb(:,:,12);
    img1_comb(:,:,13) img1_comb(:,:,14) img1_comb(:,:,15) img1_comb(:,:,16) ];

to_disp2=[ img2_comb(:,:,1) img2_comb(:,:,2) img2_comb(:,:,3) img2_comb(:,:,4) ;
    img2_comb(:,:,5) img2_comb(:,:,6) img2_comb(:,:,7) img2_comb(:,:,8) ;
    img2_comb(:,:,9) img2_comb(:,:,10) img2_comb(:,:,11) img2_comb(:,:,12);
    img2_comb(:,:,13) img2_comb(:,:,14) img2_comb(:,:,15) img2_comb(:,:,16) ];

figure;
subplot(1,2,1); imagesc(abs(to_disp1)); axis image; axis off; title('3D GRE -- mag, echo #1');
subplot(1,2,2); imagesc(angle(to_disp1)); axis image; axis off; caxis([-pi pi]); title('3D GRE -- pha, echo #1)');

figure;
subplot(1,2,1); imagesc(abs(to_disp2)); axis image; axis off; title('3D GRE -- mag, echo #2');
subplot(1,2,2); imagesc(angle(to_disp2)); axis image; axis off; caxis([-pi pi]); title('3D GRE -- pha, echo #2)');

img_1_sum=sum(abs(img1_comb),3);
roi=img_1_sum > max(img_1_sum(:))/15;
f=figure; imagesc(roi); axis image off; title('ROI');
print(f,'-djpeg','-r300','roi.jpg');
save 'roi.mat' 'roi';

fprintf('PLEASE DRAW EXLUSION REGION ON THE ROI ...\n')
roi2=roipoly();
roi=roi & roi2;
figure; imagesc(roi); axis image off; title('ROI -- manual edit');
print(f,'-djpeg','-r300','roi_manualEdit.jpg');
save 'roi_manualEdit.mat' 'roi';


% B0 3D
B0_3D=angle( img2_comb.*conj(img1_comb) )/(2*pi*delta_TE) .* repmat(roi,[1 1 nz]);
B0_3D=B0_3D(:,:,end:-1:1);

to_disp3=[ B0_3D(:,:,1) B0_3D(:,:,2) B0_3D(:,:,3) B0_3D(:,:,4) ;
    B0_3D(:,:,5) B0_3D(:,:,6) B0_3D(:,:,7) B0_3D(:,:,8) ;
    B0_3D(:,:,9) B0_3D(:,:,10) B0_3D(:,:,11) B0_3D(:,:,12);
    B0_3D(:,:,13) B0_3D(:,:,14) B0_3D(:,:,15) B0_3D(:,:,16) ];

f=figure; imagesc(to_disp3); axis image; axis off; colorbar; caxis([-100 100]);
set(gca,'FontSize',15);
print(f,'-djpeg','-r300','B0_3D.jpg');
title('B0 3D');

% compute mean B0 and through-plane gradient
lz=meas1.prot.sSliceArray.dThickness * 1e-3;  % mm -> m
dz=lz/nz;
z=([1:nz]'-0.5)/nz*lz - lz/2;
z=z(4:end-3);  % exclude a few first and last images
b0map=zeros(nx,ny);
g=zeros(nx,ny);
rsq=zeros(nx,ny);
n=1;
sbuf=[];
% figure;
cmin=min(B0_3D(:));
cmax=max(B0_3D(:));
for i=1:nx
    
    for ii=1:size(sbuf,2)
        fprintf('\b')
    end
    sbuf=sprintf('[%d/%d]',i,nx);
    fprintf(sbuf);
    
    for j=1:ny
        if roi(i,j)>0
            y=squeeze( B0_3D(i,j,4:end-3) );
            p=polyfit(z,y,1);  % linear fit
            b0map(i,j)=mean(y);
            g(i,j)=p(1);
            yfit=polyval(p,z);
            SSresid=sum( (y-yfit).^2 );
            SStotal=(length(y)-1)*var(y);
            rsq(i,j)=1-SSresid/SStotal;
            
%             if mod(n,100)==0
%                 hold off;
%                 plot(z*1000,yfit,'-b'); hold on;
%                 plot(z*1000,y,'or'); legend('FIT','DATA');
%                 xlabel('z [mm]'); ylabel('B0 [Hz]');
%                 % axis([min(z)*1e3 max(z)*1e3 cmin cmax]);
%                 title(sprintf('R^2=%f',rsq(i,j)));
%                 pause(0.1);
%             end
            
            n=n+1;
        end
    end
end
fprintf('\n');

% figure results
f1=figure; imagesc(b0map); axis image; axis off; colormap(hot); colorbar; % title('B0 [Hz]');
set(gca,'FontSize',15);
print(f1,'-djpeg','-r300','B0.jpg');

f2=figure; imagesc(g); axis image; axis off; colormap(hot); colorbar; % title('g [Hz/m]');
set(gca,'FontSize',15);
print(f2,'-djpeg','-r300','g.jpg');

f3=figure; imagesc(rsq); axis image; axis off; colormap(hot); colorbar; % title('R^2');
caxis([0 1]);
set(gca,'FontSize',15);
print(f3,'-djpeg','-r300','rsq.jpg');

B0_3D_fit=zeros(size(B0_3D));
z=([1:nz]'-0.5)/nz*lz - lz/2;
ind=find(roi>0);
[indx indy]=ind2sub(size(roi),ind);
for i=1:size(ind,1)
    B0_3D_fit( indx(i),indy(i),: )=b0map( indx(i),indy(i) ) + g( indx(i),indy(i) ) * z;
end

to_disp4=[ B0_3D_fit(:,:,1) B0_3D_fit(:,:,2) B0_3D_fit(:,:,3) B0_3D_fit(:,:,4) ;
    B0_3D_fit(:,:,5) B0_3D_fit(:,:,6) B0_3D_fit(:,:,7) B0_3D_fit(:,:,8) ;
    B0_3D_fit(:,:,9) B0_3D_fit(:,:,10) B0_3D_fit(:,:,11) B0_3D_fit(:,:,12);
    B0_3D_fit(:,:,13) B0_3D_fit(:,:,14) B0_3D_fit(:,:,15) B0_3D_fit(:,:,16) ];

figure; 
imagesc(to_disp4); axis image; axis off; caxis([-100 100]); title('B0 3D -- fit'); colorbar;
set(gca,'FontSize',15);


% correlation B0 vs. g
gamma=42.576e6;
f4=figure;
ind=find(roi>0);
x=b0map(ind);
y=g(ind);
plot(x,y,'ob'); box off; xlabel('B0 [Hz]','FontSize',15); ylabel('g [Hz/m]','FontSize',15);

hold on; 
p=polyfit(x,y,1);
yfit=p(1) * x + p(2);
plot(x,yfit,'-r');
fprintf('g=%e * B0 + %e [Hz/m]\tg=%e * B0 + %e [uT/m]\n\n',p(1),p(2),p(1)/gamma*1e6,p(2)/gamma*1e6);
set(gca,'FontSize',15);
print(f4,'-djpeg','-r300','g_vs_B0.jpg');

save 'g.mat' 'b0map' 'B0_3D' 'g' 'rsq';









